///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoExpOutStream.cc 
// -------------------
// Cego export outstream implementation
//     
// Design and Implementation by Bjoern Lemke               
//     
// (C)opyright 2000-2013 Bjoern Lemke
//
// IMPLEMENTATION MODULE
//
// Class: CegoExpOutStream
// 
// Description: 
//
// Status: QG-2.6
//
///////////////////////////////////////////////////////////////////////////////

// base includes
#include <lfcbase/Base64Coder.h>

// cego includes
#include "CegoExpOutStream.h"
#include "CegoXMLdef.h"

#include <string.h>
#include <stdlib.h>

CegoExpOutStream::CegoExpOutStream(int tabSetId, const Chain& tableName, const ListT<CegoField>& schema, CegoDistManager* pGTM, CegoAdminHandler* pAH)
{
    _pGTM = pGTM;
    _pAH = pAH;
    _tableName = tableName;
    _pOC = pGTM->getObjectCursor(tabSetId, tableName, tableName, CegoObject::TABLE);
    _schema = schema;
    _tabSetId = tabSetId;
}

CegoExpOutStream::~CegoExpOutStream()
{   
    delete _pOC;
}

Element* CegoExpOutStream::getFirst()
{
    CegoDataPointer dp;

    _rowCount = 0;
    
    if ( _pAH )
    {
	Chain info = Chain("Exporting table data for ") +  _tableName + Chain(" ...");
	_pAH->sendInfo(info + Chain("\n"));
    }

    if ( _pGTM->getFirstTuple(_pOC, _schema, dp ) )
    {
	_rowCount++;
	return getRowElement(_schema);
    }
    else
    {
	Element* pRowElement = new Element(XML_NOROWS_ELEMENT);
	return pRowElement;	
    }
}

Element* CegoExpOutStream::getNext()
{
    CegoDataPointer dp;
    if ( _pGTM->getNextTuple(_pOC, _schema, dp ) )
    {
	_rowCount++;

	if ( _pAH && ( _rowCount % XP_ROWINTERVAL == 0 ) )
	    _pAH->sendInfo(Chain(_rowCount) + Chain(" rows\r"));
	return getRowElement(_schema);
    }
    if ( _pAH )
	_pAH->sendInfo(Chain(_rowCount) + Chain(" rows exported\n"));
			   
    return 0;
}

Element* CegoExpOutStream::getRowElement(ListT<CegoField>& schema)
{

    Element* pRowElement = new Element(XML_ROW_ELEMENT);
    
    long nextBlobId=0;
    CegoField *pF = schema.First();
    while ( pF ) 
    {
	if ( ! pF->getValue().isNull() )
	{
	    if ( pF->getType() == BLOB_TYPE )
	    {
		Chain blobId = Chain(XML_BLOBPREFIX) + Chain(nextBlobId);
		nextBlobId++;
		pRowElement->setAttribute(pF->getAttrName(), blobId);
		
		int fileId;
		int pageId;
		memcpy(&fileId, pF->getValue().getValue(), sizeof(int));
		memcpy(&pageId, (void*)((long)pF->getValue().getValue() + sizeof(int)), sizeof(int));
		
		long blobSize;
		unsigned char* blobBuf = _pGTM->getBlobData(_tabSetId, fileId, pageId, blobSize);

		Base64Coder b64;
		pRowElement->addData(b64.encode(blobSize, blobBuf));
				
	    }
	    else
		pRowElement->setAttribute(pF->getAttrName(), pF->getValue().valAsChain());		
	}
	pF = _schema.Next();
    }
    return pRowElement;
}
