///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoProcAssignStmt.cc  
// ---------------------                                                     
// Cego procedure assign statement implementation
//                                                         
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2013 Bjoern Lemke
//
// IMPLEMENTATION MODULE
//
// Class: CegoProcAssignStmt
// 
// Description:
//
// Status: QG-2.6
//
///////////////////////////////////////////////////////////////////////////////

// base includes
#include <lfcbase/Exception.h>
#include <lfcbase/Tokenizer.h>
#include <lfcbase/Datetime.h>

// cego includes
#include "CegoProcAssignStmt.h"

CegoProcAssignStmt::CegoProcAssignStmt(CegoExpr *pExpr,
				       CegoProcBlock *pBlock) : CegoProcStmt(pBlock)
{
    _mode = EXPR2NULL;
    _pExpr = pExpr;
    _pExpr->setBlock(pBlock);
}

CegoProcAssignStmt::CegoProcAssignStmt(const Chain& paramName,
				       CegoExpr *pExpr,
				       CegoProcBlock *pBlock) : CegoProcStmt(pBlock)
{
    _mode = EXPR2PARAM;
    _paramName = paramName;
    _pExpr = pExpr;
    _pExpr->setBlock(pBlock);
}

CegoProcAssignStmt::~CegoProcAssignStmt()  
{
    if ( _pExpr )
    {
	delete _pExpr;    
    }
}

CegoException CegoProcAssignStmt::execute()
{    
    switch ( _mode )
    {
    case CegoProcAssignStmt::EXPR2NULL:
    {
	CegoFieldValue fv;
	fv = _pExpr->evalFieldValue();
	break;
    }
    case CegoProcAssignStmt::EXPR2PARAM:
    {
	setValue(_paramName, _pExpr->evalFieldValue());
	break;
    }
    }
    return NONE_EXCEP;
}

Chain CegoProcAssignStmt::toChain(const Chain& indent) const
{

    switch ( _mode )
    {
    case CegoProcAssignStmt::EXPR2NULL:
    {
	Chain s;
	s =  indent + _pExpr->toChain();
	return s;
    }
    case CegoProcAssignStmt::EXPR2PARAM:
    {
	Chain s;
	s = indent + Chain(":")  + _paramName + " = " + _pExpr->toChain();
	return s;
    }
    }
}
