///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoProcWhileStmt.cc  
// --------------------
// Cego procedure while look statement implementation
//               
// Design and Implementation by Bjoern Lemke
//                                   
// (C)opyright 2000-2013 Bjoern Lemke
//
// IMPLEMENTATION MODULE
//
// Class: CegoProcWhileStmt
// 
// Description: see header file for description
//
// Status: QG-2.6
//
///////////////////////////////////////////////////////////////////////////////

// base includes 
#include <lfcbase/ListT.h>

// cego includes
#include "CegoProcWhileStmt.h"

CegoProcWhileStmt::CegoProcWhileStmt(CegoProcCond *pCond,
		      CegoProcBlock *pBlock,
		      CegoProcBlock *pParentBlock) : CegoProcStmt(pParentBlock)
{
    _pCond = pCond;
    _pBlock = pBlock;
    _pCond->setBlock(_pBlock);
}

CegoProcWhileStmt::~CegoProcWhileStmt()  
{
    if ( _pBlock )
	delete _pBlock;
    if ( _pCond )
	delete _pCond;
}

void CegoProcWhileStmt::setBlock(CegoProcBlock * pBlock)
{
    _pBlock = pBlock;
}

CegoException CegoProcWhileStmt::execute()
{

    while ( _pCond->eval() )
    {
	CegoException result;
	result =  _pBlock->execute();
	if ( result != NONE_EXCEP )
	    return result;
    }
    return NONE_EXCEP;
}

Chain CegoProcWhileStmt::toChain(const Chain& indent) const
{
    Chain s;
    s = indent + Chain("while ") + _pCond->toChain() + Chain("\n") + indent + Chain("begin\n");
    s += _pBlock->toChain(indent + indent);
    s += indent + Chain("end");
    return s;
}
