#ifndef _CEGOTABLEDESC_H_INCLUDED_
#define _CEGOTABLEDESC_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoTableDesc.h
// ---------------
// Cego table description
//
// Design and Implementation by Bjoern Lemke
//
// (C)opyright 2000-2013 Bjoern Lemke
//
// INTERFACE MODULE
//
// Class: CegoTableDesc
// 
// Description: Data structure for cego table description
//
// Status: QG-2.6
//
///////////////////////////////////////////////////////////////////////////////

// base includes
#include <lfcbase/Chain.h>

// xml includes
#include <lfcxml/Element.h>

class CegoTableDesc {

 public:

    CegoTableDesc();
    CegoTableDesc(char* buf);
    CegoTableDesc(Element* pTDE);
    CegoTableDesc(const CegoTableDesc& td);
    CegoTableDesc(const Chain& tableName, const Chain& tableSet);
    CegoTableDesc(const Chain& tableName, const Chain& tableSet, const Chain& tableAlias);
    ~CegoTableDesc();
    
    void setTableName(const Chain& tableName);
    const Chain& getTableName() const;

    void setTableSet(const Chain& tableSet);
    const Chain& getTableSet() const;
    
    void setTableAlias(const Chain& tableName);
    const Chain& getTableAlias() const;
    
    CegoTableDesc& operator = ( const CegoTableDesc& ad);
    bool operator == ( const CegoTableDesc& ad);

    void encode(char *buf);
    void decode(char *buf);
    int getEncodingLength() const;

    Element* toElement() const;
    void fromElement(Element *pTableDescElement);

    Chain toChain() const;
    friend ostream& operator << (ostream& s, const CegoTableDesc& td);
    
private:
    
    Chain _tableName;
    Chain _tableSet;
    Chain _tableAlias;
    
};

#endif




